@@ -23,13 +23,12 @@
 void random_seed ( void );
 void undo_callback ( void );
 void redo_callback ( void );
-void help_about_callback ( void );
 void create_menus ( void );
-void install_menu_hints ( GnomeApp * );
 void toolbar_hide ( void );
 void undo_set_sensitive (gboolean state);
 void redo_set_sensitive (gboolean state);
 void help_update_game_name (gchar * name);
+void install_options_menu (gchar *name);
 
 extern GtkUIManager *ui_manager;
 extern GtkWidget *menubar;
