@@ -55,7 +55,7 @@
 GtkWidget        *app;
 GtkWidget        *vbox;
 GtkWidget        *playing_area;
-GtkWidget        *option_dialog = NULL;
+GtkWidget        *statusbar;
 GdkGC            *draw_gc;
 GdkGC            *bg_gc;
 GdkGC            *slot_gc;
@@ -68,6 +68,9 @@ gboolean         dont_save = FALSE; /* If the game is selected on the
                                      * This is essential for Freecell
                                      * emulation.*/
 
+gint32		 enabled_features;
+gboolean	 droppable_is_featured;
+
 guint            score;
 /* guint            game_seconds;*/
 guint            timeout;
@@ -96,9 +99,15 @@ gchar* game_file_to_name (const gchar* file)
   char* p, *buf = g_path_get_basename(file);
 
   if ((p = strrchr (buf, '.'))) *p = '\0';
-  for(p = buf; p = strchr(p, '_'), p && *p;) *p = ' ';
-  for(p = buf; p = strchr(p, '-'), p && *p;) *p = ' ';
-  buf[0] = toupper(buf[0]);
+  for (p = buf; p = strchr(p, '_'), p && *p;) *p = ' ';
+  for (p = buf; p = strchr(p, '-'), p && *p;) *p = ' ';
+  for (p = buf; p = strchr(p, ' '), p && *p;) {
+    if (*(p+1)) {
+      *(p+1) = g_ascii_toupper (*(p+1));
+      *p++;
+    }
+  }
+  buf[0] = g_ascii_toupper (buf[0]);
 
   return buf;
 }
@@ -128,18 +137,6 @@ static gchar * game_name_to_file (const gchar *name)
   return p;
 }
 
-
-void make_title () 
-{
-  char *title;
-
-  title = g_strdup_printf (_("AisleRiot: %s"), game_name);
-
-  gtk_window_set_title (GTK_WINDOW (app), title); 
-
-  g_free (title);
-}
-
 void eval_installed_file (char *file)
 {
   char *installed_filename;
@@ -223,14 +220,11 @@ void new_game (gchar* file, guint *seedp)
     game_name = game_file_to_name (file);
     update_statistics_display ();
     help_update_game_name (game_name);
+    install_options_menu (game_name);
 
     if (!dont_save)
       save_state (gnome_master_client ());
 
-    if(option_dialog) {
-      gtk_widget_destroy(option_dialog);
-      option_dialog = NULL;
-    }
   }
 
   if (seedp) {
@@ -263,7 +257,7 @@ void new_game (gchar* file, guint *seedp)
     redo_set_sensitive (FALSE);
 
     game_over = FALSE;
-    make_title();
+    gtk_window_set_title (GTK_WINDOW (app), _(game_name)); 
   }
 }
 
@@ -395,7 +389,7 @@ gchar* start_game;
 
 static void main_prog(void *closure, int argc, char *argv[])
 {
-  GtkWidget *score_label, *time_label, *score_box, *status_bar;
+  GtkWidget *score_label, *time_label, *score_box;
 
   cscm_init();
 
@@ -405,12 +399,13 @@ static void main_prog(void *closure, int argc, char *argv[])
 
   load_pixmaps ();
 
+  statusbar = gtk_statusbar_new ();
   create_menus ();
 
   gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);  
-
   create_sol_board ();
+  gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);
 
   score_box = gtk_hbox_new(0, FALSE);
   score_label = gtk_label_new (_("Score:"));
@@ -422,13 +417,12 @@ static void main_prog(void *closure, int argc, char *argv[])
   time_value = games_clock_new ();
   gtk_box_pack_start (GTK_BOX(score_box), time_value, FALSE, FALSE, GNOME_PAD_SMALL);
 
-  status_bar = gnome_appbar_new (FALSE, TRUE, FALSE);
-  gtk_box_pack_end (GTK_BOX(status_bar), score_box, FALSE, FALSE, 0);
-  gnome_app_set_statusbar (GNOME_APP (app), status_bar);
+  gtk_box_pack_end (GTK_BOX(statusbar), score_box, FALSE, FALSE, 0);
 
   new_game (start_game, NULL);
 
   gtk_widget_show_all (app);
+  gtk_window_set_focus (GTK_WINDOW (app), NULL);
 
   if (!gconf_client_get_bool (gconf_client,
                              "/apps/aisleriot/show_toolbar", NULL))
