@@ -25,6 +25,18 @@
       (eval code)
       (eval code (current-module))))
         
+;; Feature masks:
+(define droppable-feature 1)
+
+(define (reset-features) (set-feature-word! 0))
+
+(define (set-features . feature-list)
+  (set-feature-word! (+ (get-feature-word)
+		       (apply + feature-list))))
+
+;; Sol.scm gets evaluated whenever game types are switched,
+;; this makes sure that the old settings are gone.
+(reset-features)
 
 ;; Constants:
 
@@ -220,9 +232,11 @@
 ; when the stock slot is empty than the waste slot will be flipped back
 ; onto the stock unless the flip limit has been reached.
 ; an optional forth argument indicates the number of cards to deal.
+; If the flip limit is negative, it is treated as infinite.
 (define (flip-stock stock-slot waste-slot flip-limit . rest)
   (if (empty-slot? stock-slot)
-      (and (< FLIP-COUNTER flip-limit)
+      (and (or (< flip-limit 0)
+	       (< FLIP-COUNTER flip-limit))
 	   (set! FLIP-COUNTER (+ 1 FLIP-COUNTER))
 	   (flip-deck stock-slot waste-slot))
       (or (let loop ((i (if (null? rest) 1 (car rest))))
