@@ -2,7 +2,7 @@
 /*
  * Blackjack
  *
- * Copyright (C) 2003-2004 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2003-2005 William Jon McCann <mccann@jhu.edu>
  *
  * This game is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,34 +22,26 @@
 
 #include <config.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
-#include <sys/types.h>
-#include <string.h>
 #include <stdlib.h>
-#include <dirent.h>
-#include <ctype.h>
 #include <glib/gi18n.h>
-
-#include "blackjack.h"
+#include <gtk/gtk.h>
+#include <gconf/gconf-client.h>
+#include <games-stock.h>
 
 #ifdef HAVE_GNOME
 #include <gnome.h>
 #endif
 
+#include "blackjack.h"
 #include "events.h"
 #include "draw.h"
 #include "slot.h"
 #include "card.h"
 #include "menu.h"
 #include "dialog.h"
-
 #include "game.h"
 #include "hand.h"
 
-#include <iostream>
 using namespace std;
 
 // Global Variables
@@ -80,8 +72,6 @@ guint            y_spacing = 15;
 double           x_expanded_offset = 0.21;
 double           y_expanded_offset = 0.21;
 
-gboolean         events_pending = false;
-
 static GConfClient *gconf_client = NULL;
 
 gfloat wager_value = 5.0;
@@ -98,7 +88,7 @@ bj_make_window_title (gchar *game_name)
 {
         char *title;
 
-        title = g_strdup_printf (_("Blackjack: %s"), game_name);
+        title = g_strdup_printf (_("Blackjack - %s"), game_name);
 
         gtk_window_set_title (GTK_WINDOW (toplevel_window), title); 
 
@@ -262,6 +252,7 @@ create_main_window (void)
         gint          width, height;
         char         *label_string;
         GtkActionGroup *actions;
+
         static const char *ui_definition =
                 "<ui>"
                 "  <menubar name='MenuBar'>"
@@ -304,22 +295,22 @@ create_main_window (void)
 
         static GtkActionEntry entries [] = {
                 { "game-menu", NULL, N_("_Game") },
-                { "new-game", GTK_STOCK_NEW, N_("_New game"), "<Control>N", N_("Start a new game"), G_CALLBACK (on_game_new_activate) },
-                { "restart-game", GTK_STOCK_REFRESH, N_("_Restart game"), "<Control>R", N_("Restart the current game"), G_CALLBACK (on_game_restart_activate) },
-                { "show-hint", GTK_STOCK_HELP, N_("_Hint"), NULL, N_("Get a hint for the next action"), G_CALLBACK (on_game_hint_activate) },
-                { "quit-game", GTK_STOCK_QUIT, NULL, "<Control>Q", N_("Quit application"), G_CALLBACK (on_game_quit_activate) },
+                { "new-game", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (on_game_new_activate) },
+                { "restart-game", GAMES_STOCK_RESTART_GAME, NULL, NULL, NULL, G_CALLBACK (on_game_restart_activate) },
+                { "show-hint", GAMES_STOCK_HINT, NULL, NULL, NULL, G_CALLBACK (on_game_hint_activate) },
+                { "quit-game", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (on_game_quit_activate) },
                 { "settings-menu", NULL, N_("_Settings") },
-                { "show-preferences", GTK_STOCK_PREFERENCES, NULL, NULL, N_("Show game preferences"), G_CALLBACK (on_preferences_activate) },
+                { "show-preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (on_preferences_activate) },
                 { "control-menu", NULL, N_("_Control") },
-                { "hand-deal", NULL, N_("Deal"), NULL, N_("Deal a new hand"), G_CALLBACK (on_control_deal_activate) },
+                { "hand-deal", NULL, N_("D_eal"), NULL, N_("Deal a new hand"), G_CALLBACK (on_control_deal_activate) },
                 { "hand-hit", NULL, N_("_Hit"), "H", N_("Add a card to the hand"), G_CALLBACK (on_control_hit_activate) },
                 { "hand-stand", NULL, N_("_Stand"), "S", N_("Stop adding cards to the hand"), G_CALLBACK (on_control_stand_activate) },
-                { "hand-surrender", NULL, N_("Surrender"), "R", N_("Surrender the hand"), G_CALLBACK (on_control_surrender_activate) },
-                { "hand-double-down", NULL, N_("_Double down"), "D", N_("Double down"), G_CALLBACK (on_control_double_activate) },
-                { "hand-split", NULL, N_("Split the hand"), "P", N_("Split the hand"), G_CALLBACK (on_control_split_activate) },
+                { "hand-surrender", NULL, N_("S_urrender"), "R", N_("Forfeit this hand for half of your wager"), G_CALLBACK (on_control_surrender_activate) },
+                { "hand-double-down", NULL, N_("_Double down"), "D", N_("Double your wager for a single hit"), G_CALLBACK (on_control_double_activate) },
+                { "hand-split", NULL, N_("S_plit the hand"), "P", N_("Split cards in two new hands"), G_CALLBACK (on_control_split_activate) },
                 { "help-menu", NULL, N_("_Help")},
-                { "show-help-contents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Display help for this game"), G_CALLBACK (on_help_contents_activate) },
-                { "show-about", GTK_STOCK_ABOUT, NULL, NULL, N_("About this game"), G_CALLBACK (on_help_about_activate) },
+                { "show-help-contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (on_help_contents_activate) },
+                { "show-about", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (on_help_about_activate) },
         };
 
 	static GtkToggleActionEntry toggle_entries [] = {
@@ -328,7 +319,11 @@ create_main_window (void)
 
         toplevel_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
+	games_stock_init();
+	status_bar = gtk_statusbar_new ();
         ui = gtk_ui_manager_new ();
+        games_stock_prepare_for_statusbar_tooltips (ui, status_bar);
+
         gtk_ui_manager_add_ui_from_string (ui, ui_definition, -1, &error);
 
         actions = gtk_action_group_new ("Actions");
@@ -382,10 +377,7 @@ create_main_window (void)
         g_free (label_string);
         gtk_box_pack_start (GTK_BOX (group_box), balance_value_label, FALSE, FALSE, 0);
 
-
-        status_bar = gtk_statusbar_new ();
         gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 0);
-
         
         if (!gconf_client)
                 gconf_client = gconf_client_get_default ();
@@ -679,7 +671,7 @@ main (int argc, char *argv [])
                               GCONF_CLIENT_PRELOAD_NONE, NULL);
 
 #ifdef HAVE_GNOME
-        gnome_program_init ("blackjack", VERSION,
+        gnome_program_init ("Blackjack", VERSION,
                             LIBGNOMEUI_MODULE,
                             argc, argv,
                             GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
