@@ -25,6 +25,9 @@
 static GtkWidgetClass *parent_class=NULL;
 static guint gridboard_signals[LAST_SIGNAL] = { 0 };
 
+/* Save the flip pixmaps timeout so we can kill it at the games end. */
+guint timeoutid = 0;
+
 /* prototypes for private functions */
 static gint ** make_array(int width, int height); 
 static void gtk_gridboard_draw_box(GtkGridBoard *widget, gint x, gint y); 
@@ -94,7 +97,8 @@ GtkWidget * gtk_gridboard_new(gint width, gint height, char * tileset) {
         gridboard->selected=make_array(width, height);
 
         gtk_gridboard_load_pixmaps(gridboard);
-        gtk_gridboard_set_animate(gridboard, TRUE); 
+        gtk_gridboard_set_animate(gridboard, TRUE);
+        gtk_gridboard_set_visibility(gridboard, TRUE);
         
         return GTK_WIDGET(gridboard);
         
@@ -137,8 +141,12 @@ static gint gtk_gridboard_button_press(GtkWidget *widget,
    
    gridboard = GTK_GRIDBOARD (widget);
 
-   x = event->x/gridboard->tilewidth;
-   y = event->y/gridboard->tileheight;
+   x = event->x / gridboard->tilewidth;
+   y = event->y / gridboard->tileheight;
+
+   x = CLAMP (x, 0, gridboard->width - 1);
+   y = CLAMP (y, 0, gridboard->height - 1);
+
    g_signal_emit (G_OBJECT(gridboard), gridboard_signals[BOXCLICKED], 
 		  0, x, y);
 
@@ -252,6 +260,8 @@ void gtk_gridboard_paint(GtkGridBoard * gridboard) {
         int x, y;
         int piecepic;
 
+	if (!gridboard->visibility) return;
+
         for (x=0; x<gridboard->height; x++) {
                 for (y=0; y<gridboard->width; y++) {
                         piecepic=get_pixmap_num(gridboard->board[x][y]);
@@ -272,6 +282,8 @@ static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gint which, gint
  x, gint y) {
         GtkWidget * widget=GTK_WIDGET(gridboard);
         
+	if (!gridboard->visibility) return;
+
         gdk_draw_drawable (widget->window,
                 widget->style->fg_gc[widget->state],
                 gridboard->tiles_pixmap,
@@ -281,6 +293,7 @@ static void gtk_gridboard_draw_pixmap(GtkGridBoard * gridboard, gint which, gint
         /* draw lines around x, y */
         gtk_gridboard_draw_box(gridboard, x, y);
 }
+
 void gtk_gridboard_set_selection(GtkGridBoard * gridboard, gint type, 
 				 gint x, gint y) 
 {
@@ -299,6 +312,8 @@ static void gtk_gridboard_draw_box(GtkGridBoard *gridboard, gint x, gint y) {
         GdkGC * gc;
         int selected;
         static GdkColor selectcolor={12345, 30208, 41216, 33792};
+
+	if (!gridboard->visibility) return;
         
 	widget = GTK_WIDGET (gridboard);
 
@@ -391,9 +406,10 @@ static void gtk_gridboard_load_pixmaps (GtkGridBoard * gridboard) {
         }
 
         /* what happens here ? */
-        gdk_pixbuf_render_pixmap_and_mask (image, 
-                        &(gridboard->tiles_pixmap), 
-                        NULL, 127);
+        gdk_pixbuf_render_pixmap_and_mask_for_colormap (image, 
+							gdk_colormap_get_system (),
+							&(gridboard->tiles_pixmap), 
+							NULL, 127);
   
         gdk_pixbuf_unref (image);
 }
@@ -474,16 +490,24 @@ void gtk_gridboard_set_show_grid(GtkGridBoard *gridboard, gboolean showgrid) {
         gridboard->showgrid=showgrid;
         gtk_gridboard_paint(gridboard);
 }
-        
+
 void gtk_gridboard_set_animate(GtkGridBoard *gridboard, gboolean animate) {
         g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
 
         gridboard->animate=animate;
-        if (animate) { 
-                g_timeout_add(PIXMAP_FLIP_DELAY, 
-                                gtk_gridboard_flip_pixmaps,
-                                (gpointer) gridboard);
-        }
+        if (animate) {
+	  if (timeoutid)
+	    return;
+	  timeoutid = g_timeout_add(PIXMAP_FLIP_DELAY, 
+				    gtk_gridboard_flip_pixmaps,
+				    (gpointer) gridboard);
+        } else timeoutid = 0;
+}
+
+void gtk_gridboard_set_visibility(GtkGridBoard *gridboard, gboolean visibility) {
+	g_return_if_fail (GTK_IS_GRIDBOARD (gridboard));
+
+	gridboard->visibility=visibility;
 }
 
 void gtk_gridboard_set_tileset(GtkGridBoard *gridboard, gchar * tileset) {
