@@ -1,3 +1,5 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
 /* 
  *   Gnome Nibbles: Gnome Worm Game
  *   Written by Sean MacIsaac <sjm@acm.org>, Ian Peters <itp@gnu.org>
@@ -19,10 +21,14 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <libgnomeui/gnome-window-icon.h>
+#include <string.h>
 #include <gdk/gdkkeysyms.h>
 #include <time.h>
+#include <gconf/gconf-client.h>
 
+#include <games-gridframe.h>
+
+#include "main.h"
 #include "properties.h"
 #include "gnibbles.h"
 #include "worm.h"
@@ -30,14 +36,14 @@
 #include "boni.h"
 #include "preferences.h"
 #include "scoreboard.h"
+#include "network.h"
 
 GtkWidget *window;
 GtkWidget *drawing_area;
 GtkWidget *appbar;
 
 extern GdkPixmap *buffer_pixmap;
-extern GdkPixmap *gnibbles_pixmap;
-extern GdkPixmap *logo_pixmap;
+extern GdkPixbuf *logo_pixmap;
 
 GnibblesProperties *properties;
 
@@ -67,18 +73,19 @@ static struct _pointers {
 	GdkCursor *invisible;
 } pointers = { NULL };
 
-static gint main_loop (gpointer data);
 static gint add_bonus_cb (gpointer data);
-static void render_logo ();
+static void render_logo (void);
 static gint new_game_cb (GtkWidget *widget, gpointer data);
 gint pause_game_cb (GtkWidget *widget, gpointer data);
 static gint end_game_cb (GtkWidget *widget, gpointer data);
 static void quit_cb (GtkWidget *widget, gpointer data);
 static void about_cb (GtkWidget *widget, gpointer data);
-static gint show_scores_cb (GtkWidget *widget, gpointer data);
+static void show_scores_cb (GtkWidget *widget, gpointer data);
+static void new_network_game_cb(GtkWidget *widget, gpointer data);
 
 static GnomeUIInfo game_menu[] = {
 	GNOMEUIINFO_MENU_NEW_GAME_ITEM (new_game_cb, NULL),
+	GNOMEUIINFO_ITEM(N_("New Net_work Game"), NULL, new_network_game_cb, NULL),
 	GNOMEUIINFO_MENU_PAUSE_GAME_ITEM (pause_game_cb, NULL),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_SCORES_ITEM (show_scores_cb, NULL),
@@ -106,7 +113,8 @@ static GnomeUIInfo main_menu[] = {
 	GNOMEUIINFO_END
 };
 
-void hide_cursor ()
+static void
+hide_cursor (void)
 {
 	if (pointers.current != pointers.invisible) {
 		gdk_window_set_cursor (drawing_area->window, pointers.invisible);
@@ -114,7 +122,8 @@ void hide_cursor ()
 	}
 }
 
-void show_cursor ()
+static void
+show_cursor (void)
 {
 	if (pointers.current != NULL) {
 		gdk_window_set_cursor (drawing_area->window, NULL);
@@ -122,14 +131,16 @@ void show_cursor ()
 	}
 }
 
-gint game_running ()
+gint 
+game_running (void)
 {
 	return (main_id || erase_id || dummy_id || restart_id || paused);
 }
 
-static void zero_board ()
+static void
+zero_board (void)
 {
-	int i, j;
+	gint i, j;
 
 	for (i = 0; i < BOARDWIDTH; i++)
 		for (j = 0; j < BOARDHEIGHT; j++) {
@@ -138,100 +149,63 @@ static void zero_board ()
 		}
 }
 
-static gint end_game_box ()
+/* Avoid a race condition where a redraw is attempted
+ * between the window being destroyed and the destroy
+ * event being sent. */
+static gint
+delete_cb (GtkWidget * widget, gpointer data)
 {
-	gint pause_state;
-	static GtkWidget *box;
-	gint status;
+	if (main_id) g_source_remove (main_id);
+	if (erase_id) g_source_remove (erase_id);
+	if (dummy_id) g_source_remove (dummy_id);
+	if (restart_id) g_source_remove (restart_id);
 
-	if (box)
-		return 0;
-
-	pause_state = paused;
-	if (!paused)
-		pause_game_cb (NULL, (gpointer) 0);
-	box = gtk_message_dialog_new (GTK_WINDOW (window), GTK_DIALOG_MODAL,
-				      GTK_MESSAGE_QUESTION,
-				      GTK_BUTTONS_YES_NO,
-				      _("Do you really want to end this game?"));
-	status = gtk_dialog_run (GTK_DIALOG (box)) == GTK_RESPONSE_NO;
-	gtk_widget_destroy (GTK_WIDGET (box));
-	box = NULL;
-	if (!pause_state && status)
-		pause_game_cb (NULL, (gpointer) 0);
-	return (status);
+	return FALSE;
 }
 
-static void quit_cb (GtkWidget *widget, gpointer data)
+static void
+quit_cb (GtkWidget *widget, gpointer data)
 {
-	if (game_running () && end_game_box ())
-		return;
-
-	gnibbles_destroy ();
+	games_kill_server ();
 	gtk_main_quit ();
 }
 
-static void about_cb (GtkWidget *widget, gpointer data)
+static void
+about_cb (GtkWidget *widget, gpointer data)
 {
-	static GtkWidget *about;
-	GdkPixbuf *pixbuf = NULL;
-
 	const gchar *authors[] = {"Sean MacIsaac", "Ian Peters", NULL};
-	gchar *documenters[] = {
-                NULL
-        };
-        /* Translator credits */
-        gchar *translator_credits = _("translator_credits");
 
-	if (about != NULL) {
-		gdk_window_raise (about->window);
-		gdk_window_show (about->window);
-		return;
-	}
-
-	{
-		char *filename = NULL;
-
-		filename = gnome_program_locate_file (NULL,
-				GNOME_FILE_DOMAIN_APP_PIXMAP, "gnome-nibbles.png",
-				TRUE, NULL);
-		if (filename != NULL)
-		{
-			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
-			g_free (filename);
-		}
-	}
-
-	about = gnome_about_new (_("Gnibbles"), VERSION,
-			"(C) 1999 Sean MacIsaac and Ian Peters",
-			_("Send comments and bug reports to: "
-			  "sjm@acm.org, itp@gnu.org"),
-			(const char **)authors,
-			(const char **)documenters,
-			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-			pixbuf);
-	g_signal_connect (G_OBJECT (about), "destroy", G_CALLBACK
-			(gtk_widget_destroyed), &about);
-	gtk_widget_show (about);
+	gtk_show_about_dialog (GTK_WINDOW (window),
+			       "name", _("Nibbles"), 
+			       "version", VERSION,
+			       "copyright", "Copyright \xc2\xa9 1999-2004 Sean MacIsaac, Ian Peters",
+			       "comments", _("A worm game for GNOME."),
+			       "authors", authors,
+			       "translator-credits", _("translator-credits"),
+			       "logo-icon-name", "gnome-nibbles",
+			       NULL);
 }
 
 static gint expose_event_cb (GtkWidget *widget, GdkEventExpose *event)
 {
-	gdk_draw_drawable (GDK_DRAWABLE (widget->window), widget->style->fg_gc[GTK_WIDGET_STATE
-			(widget)], buffer_pixmap, event->area.x, event->area.y,
-			event->area.x, event->area.y, event->area.width,
-			event->area.height);
+	gdk_draw_drawable (GDK_DRAWABLE (widget->window),
+			   widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+			   buffer_pixmap, event->area.x, event->area.y,
+			   event->area.x, event->area.y, event->area.width,
+			   event->area.height);
 
 	return (FALSE);
 }
 
 static gint key_press_cb (GtkWidget *widget, GdkEventKey *event)
 {
-	gnibbles_keypress_worms (event->keyval);
 	hide_cursor ();
+
+	return gnibbles_keypress_worms (event->keyval);
 }
 
-static void draw_board ()
+static void
+draw_board ()
 {
 	int i, j;
 
@@ -258,21 +232,65 @@ static void draw_board ()
 		gnibbles_bonus_draw (boni->bonuses[i]);
 
 	gdk_draw_drawable (GDK_DRAWABLE (drawing_area->window),
-			 drawing_area->style->fg_gc[GTK_WIDGET_STATE
-			 (drawing_area)], buffer_pixmap, 0, 0, 0, 0,
-			 BOARDWIDTH*properties->tilesize, BOARDHEIGHT*properties->tilesize);
+			   drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
+			   buffer_pixmap, 0, 0, 0, 0,
+			   BOARDWIDTH*properties->tilesize,
+			   BOARDHEIGHT*properties->tilesize);
 	
 }
 
-static gint configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
+static gint
+window_configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
 {
+	gnibbles_properties_set_height(event->height);
+	gnibbles_properties_set_width(event->width);
+	return FALSE;
+}
+
+static gint
+configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
+{
+	int tilesize, ts_x, ts_y;
+
+	/* Compute the new tile size based on the size of the
+	 * drawing area, rounded down. */
+	ts_x = event->width / BOARDWIDTH;
+	ts_y = event->height / BOARDHEIGHT;
+	if (ts_x * BOARDWIDTH > event->width)
+		ts_x--;
+	if (ts_y * BOARDHEIGHT > event->height)
+		ts_y--;
+	tilesize = MIN(ts_x, ts_y);
+
+	/* But, has the tile size changed? */
+	if (properties->tilesize == tilesize) {
+
+		/* We must always re-load the logo. */
+		gnibbles_load_logo (window);
+		return FALSE;
+	}
+
+	properties->tilesize = tilesize;
+	gnibbles_properties_set_tile_size(tilesize);
+
+	/* Reload the images pixmap. */
+	gnibbles_load_logo (window);
 	gnibbles_load_pixmap (window);
+
+	/* Recreate the buffer pixmap. */
 	if (buffer_pixmap)
 		g_object_unref (G_OBJECT (buffer_pixmap));
 	buffer_pixmap = gdk_pixmap_new (drawing_area->window,
 					BOARDWIDTH * properties->tilesize,
 					BOARDHEIGHT * properties->tilesize, -1);
 
+	/* Erase the buffer pixmap. */
+	gdk_draw_rectangle(buffer_pixmap,
+				drawing_area->style->black_gc,
+				TRUE, 0, 0,
+				BOARDWIDTH * properties->tilesize,
+				BOARDHEIGHT * properties->tilesize);
+
 	if (game_running ())
 		draw_board ();
 	else
@@ -281,21 +299,24 @@ static gint configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
 	return (FALSE);
 }
 
-static gint new_game_2_cb (GtkWidget *widget, gpointer data)
+static gint
+new_game_2_cb (GtkWidget *widget, gpointer data)
 {
 	if (!paused) {
 		if (!keyboard_id)
 			keyboard_id = g_signal_connect (G_OBJECT (window),
-					"key_press_event",
-					G_CALLBACK (key_press_cb), NULL);
-		if (!main_id)
-			main_id = gtk_timeout_add (GAMEDELAY * properties->gamespeed,
-					(GtkFunction) main_loop, NULL);
-		if (!add_bonus_id)
-			add_bonus_id = gtk_timeout_add (BONUSDELAY *
-					properties->gamespeed,
-					(GtkFunction) add_bonus_cb,
-					NULL);
+							"key_press_event",
+							G_CALLBACK (key_press_cb),
+							NULL);
+		if (!main_id && network_is_host ())
+			main_id = g_timeout_add (GAMEDELAY * properties->gamespeed,
+						   (GSourceFunc) main_loop,
+						   NULL);
+		if (!add_bonus_id && network_is_host ()) 
+			add_bonus_id = g_timeout_add (BONUSDELAY *
+							properties->gamespeed,
+							(GSourceFunc) add_bonus_cb,
+							NULL);
 	}
 
 	dummy_id = 0;
@@ -303,29 +324,42 @@ static gint new_game_2_cb (GtkWidget *widget, gpointer data)
 	return (FALSE);
 }
 
-static gint new_game_cb (GtkWidget *widget, gpointer data)
+gint
+new_game (void)
 {
-	gtk_widget_set_sensitive (game_menu[1].widget, TRUE);
-	gtk_widget_set_sensitive (game_menu[4].widget, TRUE);
+  return new_game_cb(NULL, NULL);
+}
+
+static gint
+new_game_cb (GtkWidget *widget, gpointer data)
+{
+        gtk_widget_set_sensitive (game_menu[1].widget, FALSE);
+	if (is_network_running ()) {
+		gtk_widget_set_sensitive (game_menu[2].widget, FALSE);
+	} else {
+               gtk_widget_set_sensitive (game_menu[2].widget, TRUE);
+	}
+	gtk_widget_set_sensitive (game_menu[5].widget, TRUE);
 	/*
 	gtk_widget_set_sensitive (settings_menu[0].widget, FALSE);
 	*/
 
 	if (game_running ()) {
-		if (!end_game_box ()) {
 			end_game_cb (widget, (gpointer) 0);
 			main_id = 0;
-		} else
-			return 0;
 	}
 
 	gnibbles_init ();
 
-	if (!properties->random)
+	if (is_network_running ()) {
+		current_level = 1;
+	} else if (!properties->random) {
 		current_level = properties->startlevel;
-	else
+	} else {
 		current_level = rand () % MAXLEVEL + 1;
-	
+	}
+
+	zero_board();
 	gnibbles_load_level (GTK_WIDGET (window), current_level);
 
 	gnibbles_add_bonus (1);
@@ -333,32 +367,34 @@ static gint new_game_cb (GtkWidget *widget, gpointer data)
 	paused = 0;
 	
 	if (erase_id) {
-		gtk_timeout_remove (erase_id);
+		g_source_remove (erase_id);
 		erase_id = 0;
 	}
 
 	if (restart_id) {
-		gtk_timeout_remove (restart_id);
+		g_source_remove (restart_id);
 		restart_id = 0;
 	}
 
 	if (add_bonus_id) {
-		gtk_timeout_remove (add_bonus_id);
+		g_source_remove (add_bonus_id);
 		add_bonus_id = 0;
 	}
 
 	if (dummy_id)
-		gtk_timeout_remove (dummy_id);
+		g_source_remove (dummy_id);
 
-	dummy_id = gtk_timeout_add (1500, (GtkFunction) new_game_2_cb, NULL);
+	dummy_id = g_timeout_add (1500, (GSourceFunc) new_game_2_cb, NULL);
+
+	return TRUE;
 }
 
 gint pause_game_cb (GtkWidget *widget, gpointer data)
 {
 	if (paused) {
 		paused = 0;
-		dummy_id = gtk_timeout_add (500, (GtkFunction) new_game_2_cb,
-				NULL);
+		dummy_id = g_timeout_add (500, (GSourceFunc) new_game_2_cb,
+					  NULL);
 		/*
 		main_id = gtk_timeout_add (GAMEDELAY * properties->gamespeed,
 				(GtkFunction) main_loop, NULL);
@@ -374,7 +410,7 @@ gint pause_game_cb (GtkWidget *widget, gpointer data)
 		if (main_id || erase_id || restart_id || dummy_id ) {
 			paused = 1;
 			if (main_id) {
-				gtk_timeout_remove (main_id);
+				g_source_remove (main_id);
 				main_id = 0;
 			}
 			if (keyboard_id) {
@@ -383,25 +419,24 @@ gint pause_game_cb (GtkWidget *widget, gpointer data)
 				keyboard_id = 0;
 			}
 			if (add_bonus_id) {
-				gtk_timeout_remove (add_bonus_id);
+				g_source_remove (add_bonus_id);
 				add_bonus_id = 0;
 			}
 		}
 	}
+	return TRUE;
 }
 
-static gint show_scores_cb (GtkWidget *widget, gpointer data)
+static void show_scores_cb (GtkWidget *widget, gpointer data)
 {
-	gnibbles_show_scores (0);
+	gnibbles_show_scores (window, 0);
 }
 
-static gint end_game_cb (GtkWidget *widget, gpointer data)
+void
+end_game (gint data)
 {
-	if ((gint) data == 2 && end_game_box ())
-			return 0;
-
 	if (main_id) {
-		gtk_timeout_remove (main_id);
+		g_source_remove (main_id);
 		main_id = 0;
 	}
 
@@ -411,36 +446,44 @@ static gint end_game_cb (GtkWidget *widget, gpointer data)
 	}
 
 	if (add_bonus_id) {
-		gtk_timeout_remove (add_bonus_id);
+		g_source_remove (add_bonus_id);
 		add_bonus_id = 0;
 	}
 
 	if (erase_id) {
-		gtk_timeout_remove (erase_id);
+		g_source_remove (erase_id);
 		erase_id = 0;
 	}
 
 	if (dummy_id) {
-		gtk_timeout_remove (dummy_id);
+		g_source_remove (dummy_id);
 		dummy_id = 0;
 	}
 
 	if (restart_id) {
-		gtk_timeout_remove (restart_id);
+		g_source_remove (restart_id);
 		restart_id = 0;
 	}
 
 	if ((gint) data) {
 		render_logo ();
-		gtk_widget_set_sensitive (game_menu[1].widget, FALSE);
-		gtk_widget_set_sensitive (game_menu[4].widget, FALSE);
-		/*
+                gtk_widget_set_sensitive (game_menu[0].widget, TRUE);
+                gtk_widget_set_sensitive (game_menu[1].widget, TRUE);
+		gtk_widget_set_sensitive (game_menu[2].widget, FALSE);
+		gtk_widget_set_sensitive (game_menu[5].widget, FALSE);
 		gtk_widget_set_sensitive (settings_menu[0].widget, TRUE);
-		*/
 	}
 
 	paused = 0;
 
+	if (is_network_running ()) {
+		network_stop ();
+	}
+}
+
+static gint end_game_cb (GtkWidget *widget, gpointer data)
+{
+	end_game (GPOINTER_TO_INT(data));
 	return (FALSE);
 }
 
@@ -453,120 +496,160 @@ static gint add_bonus_cb (gpointer data)
 
 static gint restart_game (gpointer data)
 {
+	zero_board();
 	gnibbles_load_level (GTK_WIDGET (window), current_level);
 
 	gnibbles_add_bonus (1);
 
-	dummy_id = gtk_timeout_add (1500, (GtkFunction) new_game_2_cb,
-			NULL);
+	dummy_id = g_timeout_add (1500, (GSourceFunc) new_game_2_cb,
+				  NULL);
 
 	restart_id = 0;
 
 	return (FALSE);
 }
 
-static gint erase_worms_cb (gpointer data)
+static gint erase_worms_cb (gpointer datap)
 {
-	if ((gint) data == 0) {
+	gint data = GPOINTER_TO_INT(datap);
+
+	if (data == 0) {
 		erase_id = 0;
 		if (!restart_id)
-			end_game_cb (NULL, (gpointer) 1);
+			end_game_cb (NULL, GINT_TO_POINTER(1));
 	} else {
-		gnibbles_undraw_worms (ERASESIZE - (gint) data);
-		erase_id = gtk_timeout_add (ERASETIME / ERASESIZE,
-				(GtkFunction) erase_worms_cb,
-				(gpointer) ((gint) data - 1));
+		gnibbles_undraw_worms (ERASESIZE - data);
+		erase_id = g_timeout_add (ERASETIME / ERASESIZE,
+				(GSourceFunc) erase_worms_cb,
+				GINT_TO_POINTER(data - 1));
 	}
 
 	return (FALSE);
 }
 
-static gint main_loop (gpointer data)
+gint 
+main_loop (gpointer data)
 {
 	gint status;
 	gint tmp;
 
 	status = gnibbles_move_worms ();
-
 	gnibbles_scoreboard_update (scoreboard);
+	network_move_worms ();
 
 	if (status == GAMEOVER) {
 		g_signal_handler_disconnect (G_OBJECT (window), keyboard_id);
 		keyboard_id = 0;
 		main_id = 0;
-		gtk_timeout_remove (add_bonus_id);
+		if (add_bonus_id) {
+			g_source_remove (add_bonus_id);
+		}
 		add_bonus_id = 0;
-		erase_id = gtk_timeout_add (3000,
-				(GtkFunction) erase_worms_cb,
-				(gpointer) ERASESIZE);
-		gnibbles_log_score ();
+		erase_id = g_timeout_add (3000,
+					  (GSourceFunc) erase_worms_cb,
+					  (gpointer) ERASESIZE);
+		gnibbles_log_score (window);
 		return (FALSE);
 	}
 
 	if (status == NEWROUND) {
 		g_signal_handler_disconnect (G_OBJECT (window), keyboard_id);
 		keyboard_id = 0;
-		gtk_timeout_remove (add_bonus_id);
+		if (add_bonus_id) {
+			g_source_remove (add_bonus_id);
+		}
 		add_bonus_id = 0;
 		main_id = 0;
-		erase_id = gtk_timeout_add (ERASETIME / ERASESIZE,
-				(GtkFunction) erase_worms_cb,
-				(gpointer) ERASESIZE);
-		restart_id = gtk_timeout_add (1000, (GtkFunction) restart_game,
-				NULL);
+		erase_id = g_timeout_add (ERASETIME / ERASESIZE,
+					  (GSourceFunc) erase_worms_cb,
+					  (gpointer) ERASESIZE);
+		restart_id = g_timeout_add (1000, (GSourceFunc) restart_game,
+					    NULL);
 		return (FALSE);
 	}
 
 	if (boni->numleft == 0) {
 		g_signal_handler_disconnect (G_OBJECT (window), keyboard_id);
 		keyboard_id = 0;
-		gtk_timeout_remove (add_bonus_id);
+		if (add_bonus_id) {
+			g_source_remove (add_bonus_id);
+		}
 		add_bonus_id = 0;
 		main_id = 0;
-		if ((current_level < MAXLEVEL) && !properties->random)
+		if ((current_level < MAXLEVEL) && (!properties->random 
+		     || is_network_running ()))
 			current_level++;
-		else if (properties->random) {
+		else if (properties->random && !is_network_running ()) {
 			tmp = rand () % MAXLEVEL + 1;
 			while (tmp == current_level)
 				tmp = rand () % MAXLEVEL + 1;
 			current_level = tmp;
 		}
-		restart_id = gtk_timeout_add (1000, (GtkFunction) restart_game,
-				NULL);
+		restart_id = g_timeout_add (1000, (GSourceFunc) restart_game,
+					    NULL);
 		return (FALSE);
 	}
 
 	return (TRUE);
 }
 
-static void set_bg_color ()
+/*
+static void
+set_bg_color (void)
 {
-	GdkImage *tmp_image;
-	GdkColor bgcolor;
+	gdk_window_set_background (drawing_area->window,
+				   &drawing_area->style->black);
+}
+*/
 
-	tmp_image = gdk_drawable_get_image (GDK_DRAWABLE (gnibbles_pixmap), 0, 0, 1, 1);
-	bgcolor.pixel = gdk_image_get_pixel (tmp_image, 0, 0);
-	gdk_window_set_background (drawing_area->window, &bgcolor);
-	g_object_unref (G_OBJECT (tmp_image));
+void
+update_score_state (void)
+{
+        gchar **names = NULL;
+        gfloat *scores = NULL;
+        time_t *scoretimes = NULL;
+	gint top;
+
+	gchar *buf = NULL;
+	buf = g_strdup_printf ("%d.%d", properties->gamespeed,
+			       properties->fakes);
+
+	top = gnome_score_get_notable ("gnibbles", buf,
+				       &names, &scores, &scoretimes);
+	g_free (buf);
+	if (top > 0) {
+		gtk_widget_set_sensitive (game_menu[4].widget, TRUE);
+		g_strfreev (names);
+		g_free (scores);
+		g_free (scoretimes);
+	} else {
+		gtk_widget_set_sensitive (game_menu[4].widget, FALSE);
+	}
 }
 
-gboolean
+static gboolean
 show_cursor_cb (GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
         show_cursor ();
         return FALSE;
 }
 
-static void setup_window ()
+static void
+setup_window (void)
 {
-	GtkWidget *label, *hbox;
 	GdkPixmap *cursor_dot_pm;
+	GtkWidget * packing;
 
-	window = gnome_app_new ("gnibbles", "GNOME Nibbles");
-	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
+	window = gnome_app_new ("gnibbles", "Nibbles");
 	gtk_widget_realize (window);
-	g_signal_connect (G_OBJECT (window), "delete_event",
+	gtk_window_resize(GTK_WINDOW(window), properties->width, properties->height);
+	g_signal_connect (G_OBJECT (window), "destroy",
 			G_CALLBACK (quit_cb), NULL);
+	g_signal_connect (G_OBJECT (window), "delete_event",
+			G_CALLBACK (delete_cb), NULL);
+
+	packing = games_grid_frame_new (BOARDWIDTH, BOARDHEIGHT);
+	gnome_app_set_contents (GNOME_APP (window), packing);
 
 	drawing_area = gtk_drawing_area_new ();
 
@@ -580,11 +663,14 @@ static void setup_window ()
 		&drawing_area->style->fg[GTK_STATE_ACTIVE],
 		&drawing_area->style->bg[GTK_STATE_ACTIVE], 0, 0);
 
-	gnome_app_set_contents (GNOME_APP (window), drawing_area);
+	gtk_container_add (GTK_CONTAINER (packing), drawing_area);
 
 	g_signal_connect (G_OBJECT (drawing_area), "configure_event",
 			G_CALLBACK (configure_event_cb), NULL);
 
+	g_signal_connect (G_OBJECT (window), "configure_event",
+			G_CALLBACK (window_configure_event_cb), NULL);
+
 	g_signal_connect (G_OBJECT(drawing_area), "motion_notify_event",
 			G_CALLBACK (show_cursor_cb), NULL);
 
@@ -592,7 +678,8 @@ static void setup_window ()
 			G_CALLBACK (show_cursor_cb), NULL);
 
 	gtk_widget_set_size_request (GTK_WIDGET (drawing_area),
-			BOARDWIDTH*properties->tilesize, BOARDHEIGHT*properties->tilesize);
+				     BOARDWIDTH*5,
+				     BOARDHEIGHT*5);
 	g_signal_connect (G_OBJECT (drawing_area), "expose_event",
 			G_CALLBACK (expose_event_cb), NULL);
 	gtk_widget_set_events (drawing_area, GDK_BUTTON_PRESS_MASK |
@@ -608,66 +695,108 @@ static void setup_window ()
 	scoreboard = gnibbles_scoreboard_new (appbar);
 }
 
-static void load_properties ()
+static void
+gconf_key_change_cb (GConfClient *client, guint cnxn_id,
+		     GConfEntry *entry, gpointer user_data)
 {
-	properties = gnibbles_properties_new ();
+	gnibbles_properties_update (properties);
 }
 
-static void render_logo ()
+static void
+load_properties ()
 {
-	gint i, j;
+	GConfClient * client;
 
+	properties = gnibbles_properties_new ();
+	client = gconf_client_get_default ();
+
+	/* maybe this should to into properties.c */
+        gconf_client_add_dir (client,
+                              KEY_DIR,
+                              GCONF_CLIENT_PRELOAD_RECURSIVE,
+                              NULL);
+	gconf_client_notify_add (client,
+				 KEY_PREFERENCES_DIR,
+				 gconf_key_change_cb, NULL, NULL, NULL);
+}
+
+static void
+render_logo (void)
+{
 	zero_board ();
 
-	gdk_draw_drawable (GDK_DRAWABLE (buffer_pixmap),
-			drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)], logo_pixmap,
-      0, 0, 0, 0, BOARDWIDTH*properties->tilesize,
-      BOARDHEIGHT*properties->tilesize);
+	gdk_draw_pixbuf (GDK_DRAWABLE (buffer_pixmap),
+			   drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
+			   logo_pixmap,
+			   0, 0, 0, 0,
+			   BOARDWIDTH * properties->tilesize,
+			   BOARDHEIGHT * properties->tilesize,
+			   GDK_RGB_DITHER_NORMAL, 0, 0);
 
 	gdk_draw_drawable (GDK_DRAWABLE (drawing_area->window),
-			drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
-      buffer_pixmap, 0, 0, 0, 0, BOARDWIDTH*properties->tilesize,
-      BOARDHEIGHT*properties->tilesize);
+			   drawing_area->style->fg_gc[GTK_WIDGET_STATE (drawing_area)],
+			   buffer_pixmap,
+			   0, 0, 0, 0,
+			   BOARDWIDTH * properties->tilesize,
+			  BOARDHEIGHT * properties->tilesize);
 }
 
-int main (int argc, char **argv)
+int 
+main (int argc, char **argv)
 {
-	gint foo;
-
 	gnome_score_init ("gnibbles");
 
-	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain(GETTEXT_PACKAGE);
+	textdomain (GETTEXT_PACKAGE);
 	
 	gnome_program_init ("gnibbles", VERSION, LIBGNOMEUI_MODULE,
 			    argc, argv,
 			    GNOME_PARAM_POPT_TABLE, NULL,
 			    GNOME_PARAM_APP_DATADIR, REAL_DATADIR,
 			    NULL);
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-nibbles.png");
+	gtk_window_set_default_icon_name ("gnome-nibbles");
 	srand (time (NULL));
 
 	load_properties ();
 
 	setup_window ();
 
+	update_score_state ();
+
+	gnibbles_load_logo (window);
 	gnibbles_load_pixmap (window);
 
 	gtk_widget_show (window);
 
 	buffer_pixmap = gdk_pixmap_new (drawing_area->window,
-			BOARDWIDTH * properties->tilesize, BOARDHEIGHT * properties->tilesize,
-			-1);
+					BOARDWIDTH * properties->tilesize,
+					BOARDHEIGHT * properties->tilesize,
+					-1);
 
 	render_logo ();
 
-	set_bg_color ();
+	/*set_bg_color ();*/
 
-	gtk_widget_set_sensitive (game_menu[1].widget, FALSE);
-	gtk_widget_set_sensitive (game_menu[4].widget, FALSE);
+	gtk_widget_set_sensitive (game_menu[2].widget, FALSE);
+	gtk_widget_set_sensitive (game_menu[5].widget, FALSE);
 
 	gtk_main ();
 
 	return 0;
 }
+
+static void
+new_network_game_cb (GtkWidget *widget, gpointer data)
+{
+  gtk_widget_set_sensitive (settings_menu[0].widget, FALSE);
+  network_new (window);
+  gtk_widget_set_sensitive (settings_menu[0].widget, TRUE);
+}
+
+void 
+set_numworms (int num)
+{
+  properties->numworms = num;
+}
+
