@@ -26,6 +26,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gconf/gconf-client.h>
 
+#include "sound.h"
+
 extern int LINES;
 extern int COLUMNS;
 
@@ -64,10 +66,11 @@ extern int nr_of_colors;
 extern bool random_block_colors;
 extern bool do_preview;
 
-class	Field;
+class Field;
 class Preview;
-class	BlockOps;
+class BlockOps;
 class ScoreFrame;
+class HighScores;
 
 class Tetris
 {
@@ -93,6 +96,7 @@ private:
 	Preview *preview;
 	BlockOps *ops;
 	ScoreFrame *scoreFrame;
+	HighScores *high_scores;
 
 	GConfClient *gconf_client;
 	
@@ -103,15 +107,20 @@ private:
 	bool inPlay;
 
 	void generateTimer(int level);
-	
-	static gint eventHandler(GtkWidget *widget, GdkEvent *event, void *d);
+
+	static gint keyPressHandler(GtkWidget *widget, GdkEvent *event, Tetris *t);
+	static gint keyReleaseHandler(GtkWidget *widget, GdkEvent *event, Tetris *t);
 	static gchar *decodeDropData(gchar * data, gint type);
+	void saveBgOptions();
+	static void decodeColour (guint16 *data, Tetris *t);
+	static void resetColour (Tetris *t);
 	static void dragDrop(GtkWidget *widget, GdkDragContext *context,
 			     gint x, gint y, GtkSelectionData *data, 
 			     guint info, guint time, Tetris *t);
 	static int timeoutHandler(void *d);
 	static int gameQuit(GtkWidget *widget, void *d);
 	static int gameNew(GtkWidget *widget, void *d);
+	static int focusOut(GtkWidget *widget, GdkEvent *e, Tetris *t);
 	static int gamePause(GtkWidget *widget, void *d);
 	static int gameEnd(GtkWidget *widget, void *d);
 	static int gameAbout(GtkWidget *widget, void *d);
@@ -119,6 +128,7 @@ private:
 	static int gameProperties(GtkWidget *widget, void *d);
 	static void setupdialogDestroy(GtkWidget *widget, void *d);
 	static void setupdialogResponse(GtkWidget *dialog, gint response_id, void *d);
+	static void setSound (GtkWidget *widget, gpointer data);
  	static void setRotateCounterClockWise(GtkWidget *widget, void *d);
 	static void setSelectionPreview(GtkWidget *widget, void *d);
 	static void setSelectionBlocks(GtkWidget *widget, void *d);
@@ -130,6 +140,9 @@ private:
 	static void startingLevelChanged (GtkWidget *spin, gpointer data);
 	
 	static void gconfNotify (GConfClient *tmp_client, guint cnx_id, GConfEntry *tmp_entry, gpointer tmp_data);
+	static gchar *gconfGetString (GConfClient *client, const char *key, const char *default_val);
+	static int gconfGetInt (GConfClient *client, const char *key, int default_val);
+	static gboolean gconfGetBoolean (GConfClient *client, const char *key, gboolean default_val);
 	void initOptions ();
 	void setOptions ();
 	void writeOptions ();
@@ -137,6 +150,9 @@ private:
 	
 	GdkPixbuf *image;
 	GdkPixbuf *bgimage;
+	gboolean usebg;
+
+	GdkColor bgcolour;
 
 	void fillMenu(GtkWidget *menu, char *pixname, char *dirname, GList ** list, bool addnone = false);
 	
@@ -153,12 +169,19 @@ private:
 	GtkWidget *do_preview_toggle;
 	GtkWidget *random_block_colors_toggle;
 	GtkWidget *rotate_counter_clock_wise_toggle;
+	GtkWidget *sound_toggle;
+	
+	int moveLeft;
+	int moveRight;
+	int moveDown;
+	int moveDrop;
+	int moveRotate;
+	int movePause;
 	
 	GnomeUIInfo *gameMenuPtr; 
 	GnomeUIInfo *gameSettingsPtr; 
 
 	void manageFallen();
-	void showScores(gchar *title, guint pos);
 
 	bool fastFall;
 	int fastFallPoints;
