@@ -26,13 +26,14 @@
 #include <config.h>
 #include <gnome.h>
 #include <gconf/gconf-client.h>
-#include <math.h>
 #include <string.h>
+/*#include <gst/gst.h>*/
 
 #include "minefield.h"
 #include "games-clock.h"
 #include "games-frame.h"
 #include "games-scores-dialog.h"
+#include "games-stock.h"
 
 /* Limits for various minefield properties */
 #define XSIZE_MIN 4
@@ -52,37 +53,29 @@
 #define KEY_WIDTH    	       "/apps/gnomine/geometry/width"
 #define KEY_HEIGHT   	       "/apps/gnomine/geometry/height"
 
-/* So that it is only one place if we re-order the menus. */
-#define HINT_MENU_LOC 1
-
-static GtkWidget *mf_frame;
 static GtkWidget *mfield;
 static GtkWidget *pref_dialog = NULL;
-static GtkWidget *rbutton;
-static GtkWidget *ralign;
+static GtkWidget *resume_button;
+static GtkWidget *resume_container;
+static GtkWidget *mfield_container;
 static GConfClient *conf_client;
 GtkWidget *window;
 GtkWidget *flabel;
-GtkWidget *xentry;
-GtkWidget *yentry;
 GtkWidget *mentry;
-GtkWidget *sentry;
-GtkWidget *question_toggle;
 GtkWidget *mbutton;
-GtkWidget *plabel;
 GtkWidget *cframe;
 GtkWidget *clk;
 GtkWidget *pm_win, *pm_sad, *pm_smile, *pm_cool, *pm_worried, *pm_current;
-GtkWidget *face_box;
 gint ysize = -1, xsize = -1;
 gint nmines = -1;
 gint fsize = -1;
 gboolean use_question_marks = TRUE;
 
-GnomeUIInfo gamemenu[];
-GnomeUIInfo settingsmenu[];
-GnomeUIInfo helpmenu[];
-GnomeUIInfo mainmenu[];
+GtkAction *hint_action;
+GtkAction *fullscreen_action;
+GtkAction *leavefullscreen_action;
+
+/*GstElement *sound_player;*/
 
 char *fsize2names[] = {
 	N_("Small"),
@@ -95,13 +88,20 @@ char *fsize2names[] = {
  * hide-the-window-to-stop cheating thing. */
 gboolean disable_hiding = FALSE;
 
-static void create_preferences (void);
+#if 0
+static void
+play_sound (int id)
+{
+	/* FIXME: We ignore the id */
 
+	/* To play, do this: */
+	gst_element_set_state (sound_player, GST_STATE_PLAYING);	
+}
+#endif
 
 static GtkWidget *
 image_widget_setup (char *name)
 {
-	/*	GdkPixbuf *pixbuf = NULL; */
 	GtkWidget *image = NULL;
 	char *filename = NULL;
 
@@ -111,11 +111,9 @@ image_widget_setup (char *name)
 					      TRUE, NULL);
 	if (filename != NULL)
 		gtk_image_set_from_file (GTK_IMAGE (image), filename);
-	/*		pixbuf = gdk_pixbuf_new_from_file (filename, NULL); */
 
 	g_free (filename);
 
-	/*	return gtk_image_new_from_pixbuf (pixbuf); */
 	return image;
 }
 
@@ -127,17 +125,15 @@ show_face (GtkWidget *pm)
 	if (pm_current) {
 		gtk_widget_hide (pm_current); 
 	}
-
-	gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget, 
+	gtk_action_set_sensitive(hint_action,
 				 (pm == pm_cool) || (pm == pm_smile));
-
 	gtk_widget_show (pm);
 	
 	pm_current = pm;
 }
 
 static void
-quit_game (GtkWidget *widget, gpointer data)
+quit_game (void)
 {
 	gint width, height;
 
@@ -153,58 +149,112 @@ set_flabel (GtkMineField *mfield)
 {
 	char *val;
 
-	val = g_strdup_printf ("%d/%d", mfield->flag_count, mfield->mcount);
+	val = g_strdup_printf (_("Flags: %d/%d"), mfield->flag_count, mfield->mcount);
 	gtk_label_set_text (GTK_LABEL (flabel), val);
 	g_free (val);
 }
 
-static void
-show_scores (gchar *level, guint pos)
+/* Show the high scores dialog - creating it if necessary. If pos is 
+ * greater than 0 the appropriate score is highlighted. If the score isn't
+ * a high score and this isn't a direct request to see the scores, we 
+ * only show a simple dialog. */
+static gint
+show_scores (gchar *level, gint pos, gboolean endofgame)
 {
 	int i;
-	static GtkWidget *dialog = NULL;
+	gchar *message;
+	static GtkWidget *scoresdialog = NULL;
+	static GtkWidget *sorrydialog = NULL;
+	GtkWidget *dialog;
+	gint result;
 
-	if (dialog != NULL) {
-		gtk_window_present (GTK_WINDOW (dialog));
-	} else {
-		dialog = games_scores_dialog_new ("gnomine", _("GNOME Mines Scores"));
-		for (i=0; i<4; i++) {
-			games_scores_dialog_add_category (GAMES_SCORES_DIALOG (dialog),
-							  fsize2names[i],
-							  _(fsize2names[i]));
+	if (endofgame && (pos <= 0)) {
+		if (sorrydialog != NULL) {
+			gtk_window_present (GTK_WINDOW (sorrydialog));
+		} else {
+			sorrydialog = gtk_message_dialog_new_with_markup (
+					  GTK_WINDOW (window),
+					  GTK_DIALOG_DESTROY_WITH_PARENT,
+					  GTK_MESSAGE_INFO,
+					  GTK_BUTTONS_NONE, 
+					  "<b>%s</b>\n%s", 
+					  _("The Mines Have Been Cleared!"), 
+					  _("Great work, but unfortunately your score did not make the top ten."));
+			gtk_dialog_add_buttons (GTK_DIALOG (sorrydialog),
+						GTK_STOCK_QUIT, GTK_RESPONSE_REJECT,
+						_("New Game"),
+						GTK_RESPONSE_ACCEPT,
+						NULL);
+			gtk_dialog_set_default_response (GTK_DIALOG (sorrydialog),
+							 GTK_RESPONSE_ACCEPT);
+			gtk_window_set_title (GTK_WINDOW (sorrydialog), "");
 		}
-		games_scores_dialog_set_category (GAMES_SCORES_DIALOG (dialog),
-						  fsize2names[fsize]);
-		games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG (dialog),
-							      _("Size:"));	
-		games_scores_dialog_set_style (GAMES_SCORES_DIALOG (dialog),
-					       GAMES_SCORES_STYLE_TIME);
-	}
+		dialog = sorrydialog;
+	} else {
 
-	if (pos > 0) {
-		games_scores_dialog_set_category (GAMES_SCORES_DIALOG (dialog),
-						   level);
-		games_scores_dialog_set_hilight (GAMES_SCORES_DIALOG (dialog), 
-						 pos);
-	}
+		if (scoresdialog != NULL) {
+			gtk_window_present (GTK_WINDOW (scoresdialog));
+		} else {
+			scoresdialog = games_scores_dialog_new ("gnomine", _("Mines Scores"));
+			for (i=0; i<4; i++) {
+				games_scores_dialog_add_category (GAMES_SCORES_DIALOG (scoresdialog),
+								  fsize2names[i],
+								  _(fsize2names[i]));
+			}
+			games_scores_dialog_set_category (GAMES_SCORES_DIALOG (scoresdialog),
+							  fsize2names[fsize]);
+			games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG (scoresdialog),
+								      _("Size:"));	
+			games_scores_dialog_set_style (GAMES_SCORES_DIALOG (scoresdialog),
+						       GAMES_SCORES_STYLE_TIME);
+		}
+		
+		games_scores_dialog_set_category (GAMES_SCORES_DIALOG (scoresdialog),
+						  level);
+		if (pos > 0) {
+			games_scores_dialog_set_hilight (GAMES_SCORES_DIALOG (scoresdialog), 
+							 pos);
+			message = g_strdup_printf ("<b>%s</b>\n\n%s", 
+						   _("Congratulations!"),
+						   _("Your score has made the top ten."));
+			games_scores_dialog_set_message (GAMES_SCORES_DIALOG (scoresdialog),
+							 message);
+			g_free (message);
+		} else {
+			games_scores_dialog_set_message (GAMES_SCORES_DIALOG (scoresdialog),
+							 NULL);
+		}
 
-	gtk_dialog_run (GTK_DIALOG (dialog));
+		if (endofgame) {
+			games_scores_dialog_set_buttons (GAMES_SCORES_DIALOG (scoresdialog),
+							 GAMES_SCORES_QUIT_BUTTON |
+							 GAMES_SCORES_NEW_GAME_BUTTON);
+		} else {
+			games_scores_dialog_set_buttons (GAMES_SCORES_DIALOG (scoresdialog),
+							 0);
+		}
+		dialog = scoresdialog;
+	}
+		
+	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_hide (dialog);
+
+	return result;
 }
 
 static void
-top_ten (GtkWidget *widget, gpointer data)
+scores_callback (void)
 {
 	gchar buf[64];
 
 	if(fsize<4)
 		strncpy (buf, fsize2names[fsize], sizeof(buf));
 
-	show_scores (buf, 0);
+	show_scores (buf, 0, FALSE);
 }
 
 static void
-new_game (GtkWidget *widget, gpointer data)
+new_game (void)
 {
 	gint width, height, w_diff, h_diff;
 	guint size;
@@ -236,8 +286,8 @@ new_game (GtkWidget *widget, gpointer data)
 
 	set_flabel (GTK_MINEFIELD (mfield));
 
-	gtk_widget_hide (ralign);
-	gtk_widget_show (mf_frame); 
+	gtk_widget_hide (resume_container);
+	gtk_widget_show (mfield_container); 
 }
 
 /* Add a penalty for a successful hint. */
@@ -249,7 +299,7 @@ hint_used (GtkWidget *widget, gpointer data)
 }
 
 static void
-hint (GtkWidget *widget, gpointer data)
+hint_callback (void)
 {
 	int result;
 	gchar * message;
@@ -284,11 +334,11 @@ focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
 	if ((GAMES_CLOCK (clk)->timer_id != -1) 
 	    && (!disable_hiding)) {
-		gtk_widget_hide (mf_frame);
-		gtk_widget_show (ralign);
-		gtk_widget_grab_focus (rbutton);
-		gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget,FALSE);
-
+		gtk_widget_hide (mfield_container);
+		gtk_widget_show (resume_container);
+		gtk_widget_grab_focus (resume_button);
+		
+		gtk_action_set_sensitive (hint_action, FALSE);
 		games_clock_stop (GAMES_CLOCK (clk)); 
 	}
 }
@@ -296,10 +346,9 @@ focus_out_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 static void
 resume_game_cb (GtkButton *widget, gpointer data)
 {
-	gtk_widget_hide (ralign);
-	gtk_widget_show (mf_frame);
-	gtk_widget_set_sensitive(gamemenu[HINT_MENU_LOC].widget,TRUE);
-
+	gtk_widget_hide (resume_container);
+	gtk_widget_show (mfield_container);
+	gtk_action_set_sensitive (hint_action, TRUE);
 	games_clock_start (GAMES_CLOCK (clk));
 }
 
@@ -347,7 +396,10 @@ win_game (GtkWidget *widget, gpointer data)
 	    pos = gnome_score_log (score, fsize2names[fsize], TRUE);
 	}
 
-	show_scores (fsize2names[fsize], pos);
+	if (show_scores (fsize2names[fsize], pos, TRUE) == GTK_RESPONSE_REJECT)
+		quit_game ();
+	else
+		new_game ();
 }
 
 static void
@@ -369,17 +421,14 @@ verify_ranges (void)
 {
 	xsize    = CLAMP (xsize, XSIZE_MIN, XSIZE_MAX);
 	ysize    = CLAMP (ysize, YSIZE_MIN, YSIZE_MAX);
-	nmines   = CLAMP (nmines, 1, xsize * ysize - 2);
+	nmines   = CLAMP (nmines, 1, xsize * ysize - 9);
 	fsize    = CLAMP (fsize,  0, 3);
 }
 
 static void
-about (GtkWidget *widget, gpointer data)
+about_callback (void)
 {
-        static GtkWidget *about;
-	GdkPixbuf *pixbuf = NULL;
-
-        const gchar *authors[] = {
+	const gchar *authors[] = {
 		_("Main game:"),
 		"Szekeres Istvan",
 		"",
@@ -397,49 +446,15 @@ about (GtkWidget *widget, gpointer data)
 		"Callum McKenzie",
 		NULL
 	};
-	const gchar *documenters[] = {
-                NULL
-        };
-	const gchar *translator_credits = _("translator-credits");
-	
-	if (about) {
-                gtk_window_present (GTK_WINDOW (about));
-		return;
-	}
 
-       {
-            int i=0;
-            while (authors[i] != NULL) { authors[i]=_(authors[i]); i++; }
-       }
-       {
-	       char *filename = NULL;
-
-	       filename = gnome_program_locate_file (NULL,
-			       GNOME_FILE_DOMAIN_APP_PIXMAP,  ("gnome-gnomine.png"),
-			       TRUE, NULL);
-	       if (filename != NULL)
-	       {
-		       pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
-		       g_free (filename);
-	       }
-       }
-
-        about = gnome_about_new (_("GNOME Mines"), VERSION,
-				 "Copyright \xc2\xa9 1997-2004 Free Software "
-				 "Foundation, Inc.",
-				 _("A Minesweeper clone."),
-				 (const char **)authors,
-				 (const char **)documenters,
-				 strcmp (translator_credits, "translator-credits") != 0 ? translator_credits : NULL,
-				 pixbuf);
-	
-	if (pixbuf != NULL)
-		g_object_unref (pixbuf);
-	
-	g_signal_connect (GTK_OBJECT (about), "destroy", GTK_SIGNAL_FUNC
-			(gtk_widget_destroyed), &about);
-	gtk_window_set_transient_for(GTK_WINDOW (about), GTK_WINDOW (window));
-        gtk_widget_show (about);
+	gtk_show_about_dialog (GTK_WINDOW(window),
+			       "name", _("Mines"),
+			       "version", VERSION,
+			       "copyright", "Copyright \xc2\xa9 1997-2004 Free Software Foundation, Inc.",
+			       "authors", authors,
+			       "translator_credits", _("translator-credits"),
+			       "logo-icon-name", "gnome-gnomine",
+			       NULL);
 }
 
 static void
@@ -457,7 +472,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 16;
 		if (i != xsize) {
 			xsize = CLAMP (i, XSIZE_MIN, XSIZE_MAX);
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_YSIZE) == 0) {
@@ -465,7 +480,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 16;
 		if (i != ysize) {
 			ysize = CLAMP (i, YSIZE_MIN, YSIZE_MAX);
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_NMINES) == 0) {
@@ -473,7 +488,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 40;
 		if (nmines != i) {
 			nmines = CLAMP (i, 1, xsize * ysize - 2);
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_MODE) == 0) {
@@ -481,7 +496,7 @@ gconf_key_change_cb (GConfClient *client, guint cnxn_id,
 		i = value ? gconf_value_get_int (value) : 0;
 		if (i != fsize) {
 			fsize = CLAMP (i, 0, 3);
-			new_game (mfield, NULL);
+			new_game ();
 		}
 	}
 	if (strcmp (key, KEY_USE_QUESTION_MARKS) == 0) {
@@ -510,10 +525,11 @@ fix_nmines (int xsize, int ysize)
 {
 	int maxmines;
 
-	/* Fix up the maximum number of mines so that there is always at least two
-	 * free spaces. It could in theory be left at one, but that gives an
-	 * instant win situation. */
-	maxmines = xsize*ysize - 2;
+	/* Fix up the maximum number of mines so that there is always at least
+	   ten free spaces. Nine are so we can clear at least the immediate
+	   eight neighbours at the start and one more so the game isn't over 
+	   immediately. */
+	maxmines = xsize*ysize - 10;
 	if (nmines > maxmines) {
 		gconf_client_set_int (conf_client, KEY_NMINES, maxmines, NULL);
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (mentry), maxmines);
@@ -556,6 +572,34 @@ use_question_toggle_cb (GtkCheckButton *check, gpointer data)
 				use_marks, NULL);
 }
 
+static void
+set_fullscreen_actions (gboolean is_fullscreen)
+{
+	gtk_action_set_sensitive (leavefullscreen_action, is_fullscreen);
+	gtk_action_set_visible (leavefullscreen_action, is_fullscreen);
+
+	gtk_action_set_sensitive (fullscreen_action, !is_fullscreen);
+	gtk_action_set_visible (fullscreen_action, !is_fullscreen);
+}
+
+static void
+fullscreen_callback (GtkAction *action)
+{
+	if (action == fullscreen_action)
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	else
+		gtk_window_unfullscreen (GTK_WINDOW (window));
+}
+  	 
+static void
+window_state_callback (GtkWidget *widget, GdkEventWindowState *event)
+{
+	if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
+		return;
+
+	set_fullscreen_actions (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN);
+}
+
 static void
 create_preferences (void)
 {
@@ -565,16 +609,18 @@ create_preferences (void)
 	GtkWidget *button;
 	GtkWidget *table2;
 	GtkWidget *label2;
-        
-	table = gtk_table_new (2, 3, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD);
-	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD);
-	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD);
+	GtkWidget *question_toggle;        
+	GtkWidget *xentry;
+	GtkWidget *yentry;
 
-	frame = games_frame_new (_("Field size"));
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 18);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 18);
 
-	vbox = gtk_vbox_new (TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
+	frame = games_frame_new (_("Field Size"));
+
+	vbox = gtk_vbox_new (FALSE, 6);
 
 	button = gtk_radio_button_new_with_label(NULL, _("Small"));
 	if (fsize == 0)
@@ -612,80 +658,77 @@ create_preferences (void)
 				TRUE);
 	g_signal_connect (GTK_OBJECT (button), "clicked",
 			GTK_SIGNAL_FUNC (size_radio_callback), (gpointer) 3);
-	gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 
-	gtk_table_attach (GTK_TABLE (table), frame, 0, 1, 0, 1, GTK_EXPAND |
-			GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 1, 0, 1);
 
-	cframe = games_frame_new (_("Custom size"));
+	cframe = games_frame_new (_("Custom Size"));
 	gtk_widget_set_sensitive (cframe, fsize == 3);
 
 	table2 = gtk_table_new (3, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (table2), GNOME_PAD);
-	gtk_table_set_row_spacings (GTK_TABLE (table2), GNOME_PAD);
+	gtk_table_set_row_spacings (GTK_TABLE (table2), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table2), 12);
 
-	label2 = gtk_label_new (_("Number of mines:"));
+	label2 = gtk_label_new_with_mnemonic (_("_Number of mines:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 2, 3,
-			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	
 	mentry = gtk_spin_button_new_with_range (1, XSIZE_MAX*YSIZE_MAX, 1);
 	g_signal_connect (GTK_OBJECT (mentry), "value-changed",
 			GTK_SIGNAL_FUNC (nmines_spin_cb), NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(mentry), nmines);
-	gtk_table_attach (GTK_TABLE (table2), mentry, 1, 2, 2, 3, GTK_FILL, GTK_EXPAND
-			| GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), mentry, 1, 2, 2, 3, 0, 0, 0, 0);
 	fix_nmines (xsize, ysize);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), mentry);
 
-	label2 = gtk_label_new (_("Horizontal:"));
+	label2 = gtk_label_new_with_mnemonic (_("_Horizontal:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1,
-			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
 	xentry = gtk_spin_button_new_with_range (XSIZE_MIN, XSIZE_MAX, 1);
 	g_signal_connect (GTK_OBJECT (xentry), "value-changed",
 			GTK_SIGNAL_FUNC (xsize_spin_cb), NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(xentry), xsize);
-	gtk_table_attach (GTK_TABLE (table2), xentry, 1, 2, 0, 1, 0, GTK_EXPAND
-			| GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), xentry, 1, 2, 0, 1, 0, 0, 0, 0);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), xentry);
 
-	label2 = gtk_label_new (_("Vertical:"));
+	label2 = gtk_label_new_with_mnemonic (_("_Vertical:"));
 	gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);
-	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 1, 2,
-			GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), label2, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
 	yentry = gtk_spin_button_new_with_range (YSIZE_MIN, YSIZE_MAX, 1);
 	g_signal_connect (GTK_OBJECT (yentry), "value-changed",
 			GTK_SIGNAL_FUNC (ysize_spin_cb), NULL);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(yentry), ysize);
-	gtk_table_attach (GTK_TABLE (table2), yentry, 1, 2, 1, 2, 0, GTK_EXPAND
-			| GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table2), yentry, 1, 2, 1, 2, 0, 0, 0, 0); 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label2), yentry);
 
 	gtk_container_add (GTK_CONTAINER (cframe), table2);
 
-	gtk_table_attach (GTK_TABLE (table), cframe, 1, 2, 0, 1, GTK_EXPAND |
-			GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_table_attach (GTK_TABLE (table), cframe, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0); 
 
-	question_toggle = gtk_check_button_new_with_label(_("Use \"I'm not sure\" flags."));
+	question_toggle = gtk_check_button_new_with_mnemonic(_("_Use \"I'm not sure\" flags"));
 	g_signal_connect(GTK_OBJECT(question_toggle), "toggled",
 			GTK_SIGNAL_FUNC (use_question_toggle_cb), NULL);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(question_toggle), use_question_marks);
 	gtk_widget_show(question_toggle);
 	
-	gtk_table_attach (GTK_TABLE (table), question_toggle, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_table_attach_defaults (GTK_TABLE (table), question_toggle, 0, 2, 1, 2);
 
 
-	pref_dialog = gtk_dialog_new_with_buttons (_("GNOME Mines Preferences"),
+	pref_dialog = gtk_dialog_new_with_buttons (_("Mines Preferences"),
 			GTK_WINDOW (window),
 			0,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			NULL);
 
         gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
-	gtk_container_add (GTK_CONTAINER(GTK_DIALOG(pref_dialog)->vbox),
-			table);
+	gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), 2);
+	gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(pref_dialog)->vbox), table, FALSE, FALSE, 0);
 
 	g_signal_connect (G_OBJECT (pref_dialog), "destroy",
 			  G_CALLBACK (gtk_widget_destroyed), &pref_dialog);
@@ -697,40 +740,78 @@ create_preferences (void)
 }
 
 static void
-preferences_callback (GtkWidget *widget, gpointer data)
+preferences_callback (void)
 {
 	if (pref_dialog == NULL)
 		create_preferences ();
 	gtk_window_present (GTK_WINDOW (pref_dialog));
 }
 
-GnomeUIInfo gamemenu[] = {
-        GNOMEUIINFO_MENU_NEW_GAME_ITEM(new_game, NULL),
-	GNOMEUIINFO_MENU_HINT_ITEM(hint, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_SCORES_ITEM(top_ten, NULL),
-	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_MENU_QUIT_ITEM(quit_game, NULL),
-	GNOMEUIINFO_END
+static void
+help_callback (void)
+{
+	gnome_help_display ("gnomine.xml", NULL, NULL);
+}
+
+const GtkActionEntry actions[] = {
+	{ "GameMenu", NULL, N_("_Game") },
+	{ "SettingsMenu", NULL, N_("_Settings") },
+	{ "HelpMenu", NULL, N_("_Help") },
+	{ "NewGame", GAMES_STOCK_NEW_GAME, NULL, NULL, NULL, G_CALLBACK (new_game) },
+	{ "Hint", GAMES_STOCK_HINT, NULL, NULL , NULL, G_CALLBACK (hint_callback) },
+	{ "Scores", GAMES_STOCK_SCORES, NULL, NULL, NULL, G_CALLBACK (scores_callback) },
+	{ "Quit", GTK_STOCK_QUIT, NULL, NULL, NULL, G_CALLBACK (quit_game) },
+	{ "Preferences", GTK_STOCK_PREFERENCES, NULL, NULL, NULL, G_CALLBACK (preferences_callback) },
+	{ "Contents", GAMES_STOCK_CONTENTS, NULL, NULL, NULL, G_CALLBACK (help_callback) },
+	{ "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK (about_callback) },
+	{ "Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) },
+	{ "LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL, G_CALLBACK (fullscreen_callback) }
 };
 
-GnomeUIInfo settingsmenu[] = {
-	GNOMEUIINFO_MENU_PREFERENCES_ITEM(preferences_callback, NULL),
-	GNOMEUIINFO_END
-};
+const char *ui_description =
+"<ui>"
+"  <menubar name='MainMenu'>"
+"    <menu action='GameMenu'>"
+"      <menuitem action='NewGame'/>"
+"      <menuitem action='Hint'/>"
+"      <separator/>"
+"      <menuitem action='Scores'/>"
+"      <separator/>"
+"      <menuitem action='Quit'/>"
+"    </menu>"
+"    <menu action='SettingsMenu'>"
+"      <menuitem action='Fullscreen'/>"
+"      <menuitem action='LeaveFullscreen'/>"
+"      <menuitem action='Preferences'/>"
+"    </menu>"
+"    <menu action='HelpMenu'>"
+"      <menuitem action='Contents'/>"
+"      <menuitem action='About'/>"
+"    </menu>"
+"  </menubar>"
+"</ui>";
 
-GnomeUIInfo helpmenu[] = {
-        GNOMEUIINFO_HELP("gnomine"),
-	GNOMEUIINFO_MENU_ABOUT_ITEM(about, NULL),
-	GNOMEUIINFO_END
-};
+static GtkUIManager*
+create_ui_manager (const gchar *group)
+{
+	GtkActionGroup *action_group;
+	GtkUIManager *ui_manager;
+	
+	action_group = gtk_action_group_new ("group");
+	gtk_action_group_set_translation_domain(action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group, actions, G_N_ELEMENTS (actions), window);
 
-GnomeUIInfo mainmenu[] = {
-        GNOMEUIINFO_MENU_GAME_TREE(gamemenu),
-	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
-        GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
-	GNOMEUIINFO_END
-};
+	ui_manager = gtk_ui_manager_new ();
+	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
+	hint_action = gtk_action_group_get_action (action_group, "Hint");
+
+	fullscreen_action = gtk_action_group_get_action (action_group, "Fullscreen");
+	leavefullscreen_action = gtk_action_group_get_action (action_group, "LeaveFullscreen");
+	set_fullscreen_actions (FALSE);
+
+	return ui_manager;
+}
 
 static int
 save_state (GnomeClient        *client,
@@ -785,6 +866,29 @@ static struct poptOption options[] = {
   {NULL, '\0', 0, NULL, 0}
 };
 
+#if 0
+static void 
+sound_eos (GstElement *sound_player, gpointer data)
+{
+	/* Reset the sound. */
+	gst_element_set_state (sound_player, GST_STATE_NULL);
+}
+
+static void 
+sound_init (int * argcp, char ** argvp[])
+{
+
+	gst_init (argcp, argvp);
+	sound_player = gst_element_factory_make ("playbin", "play");
+	g_object_set (G_OBJECT (sound_player), "uri", "file:///home/callum/Desktop/Development/CVS/gnome/gnome-games/gnometris/sounds/land.wav", NULL);
+
+	/* Attach the signal for end-of-stream. */
+	g_signal_connect (G_OBJECT (sound_player), "eos", G_CALLBACK (sound_eos), 
+			  NULL);
+
+}
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -793,7 +897,11 @@ main (int argc, char *argv[])
 	GtkWidget *button_table;
         GtkWidget *box;        
         GtkWidget *label;
+	GtkWidget *face_box;
 	GnomeClient *client;
+	GtkUIManager *ui_manager;
+	GtkAccelGroup *accel_group;
+
 	gint width, height;
 
 	gnome_score_init("gnomine");
@@ -808,7 +916,9 @@ main (int argc, char *argv[])
 			GNOME_PARAM_POPT_TABLE, options,
 			GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
-	/* Get the default GConfClient */
+	/* sound_init (&argc, &argv); */
+
+	/* We hold one global reference to the default gconf client. */
 	conf_client = gconf_client_get_default ();
         gconf_client_add_dir (conf_client, 
                               KEY_DIR,
@@ -853,8 +963,9 @@ main (int argc, char *argv[])
 	 * to be the default). */
 	gtk_rc_parse_string ("style \"gnomine\" { GtkButton::interior-focus = 0 } class \"GtkButton\" style \"gnomine\"");
 
-	window = gnome_app_new ("gnomine", _("GNOME Mines"));
-	gnome_app_create_menus (GNOME_APP (window), mainmenu);
+	window = gnome_app_new ("gnomine", _("Mines"));
+	games_stock_init ();
+
 	gtk_window_set_default_size (GTK_WINDOW (window), width, height);	
 	gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
 
@@ -862,13 +973,20 @@ main (int argc, char *argv[])
 			 G_CALLBACK (quit_game), NULL);
 	g_signal_connect(G_OBJECT (window), "focus_out_event",
 			 G_CALLBACK (focus_out_cb), NULL);
+	g_signal_connect (G_OBJECT (window), "window_state_event",
+			  G_CALLBACK (window_state_callback), NULL);
 
 	all_boxes = gtk_vbox_new (FALSE, 0);
 
 	gnome_app_set_contents (GNOME_APP (window), all_boxes);
+        ui_manager = create_ui_manager ("GnomineActions");
+        accel_group = gtk_ui_manager_get_accel_group (ui_manager);
+        gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+        box = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
+        gtk_box_pack_start (GTK_BOX (all_boxes), box, FALSE, FALSE, 0);
 
 	button_table = gtk_table_new (1, 3, FALSE);
-	gtk_box_pack_start (GTK_BOX (all_boxes), button_table, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (all_boxes), button_table, TRUE, TRUE, 0);
 
 	pm_current = NULL;
 
@@ -901,21 +1019,20 @@ main (int argc, char *argv[])
 
 	mfield = gtk_minefield_new ();
 
-	/* It doesn't really matter what this widget is as long as it's a
-	 * container. */
-	mf_frame = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
-	gtk_container_add (GTK_CONTAINER (mf_frame), mfield);
+	/* These next two widgets are created as alignments, but it's more
+	 * important to know that these are containers, hence the names. */
+	mfield_container = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
+	gtk_container_add (GTK_CONTAINER (mfield_container), mfield);
+	gtk_box_pack_start (GTK_BOX (box), mfield_container, TRUE, TRUE, 0);
 
-	gtk_box_pack_start (GTK_BOX (box), mf_frame, TRUE, TRUE, 0);
+	resume_container = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (box), resume_container, TRUE, TRUE, 0);
 
-	ralign = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-	gtk_box_pack_start (GTK_BOX (box), ralign, TRUE, TRUE, 0);
-
-	rbutton = gtk_button_new_with_label ("Press to resume");
-	g_signal_connect (G_OBJECT (rbutton), "clicked", 
+	resume_button = gtk_button_new_with_label (_("Press to Resume"));
+	g_signal_connect (G_OBJECT (resume_button), "clicked", 
 			  G_CALLBACK (resume_game_cb), NULL);
-        gtk_container_add (GTK_CONTAINER (ralign), rbutton);
-
+        gtk_container_add (GTK_CONTAINER (resume_container), resume_button);
+	
 	gtk_minefield_set_use_question_marks (GTK_MINEFIELD (mfield),
 					      use_question_marks);
 	
@@ -939,23 +1056,23 @@ main (int argc, char *argv[])
 	gtk_box_pack_start (GTK_BOX (box), status_box, 
 			    FALSE, FALSE, GNOME_PAD);
 
-	label = gtk_label_new(_("Flags:"));
-	gtk_box_pack_start (GTK_BOX (status_box), label, 
-			    FALSE, FALSE, 0); 
-	
-	flabel = gtk_label_new ("0");
+	flabel = gtk_label_new ("");
 	gtk_box_pack_start (GTK_BOX (status_box), flabel, 
 			    FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("Time:"));
-	gtk_box_pack_start (GTK_BOX (status_box), label, 
+	box = gtk_hbox_new (FALSE, 0);
+	label = gtk_label_new (_("Time: "));
+	gtk_box_pack_start (GTK_BOX (box), label, 
 	FALSE, FALSE, 0);
 
         clk = games_clock_new ();
-	gtk_box_pack_start (GTK_BOX (status_box), clk, 
+	gtk_box_pack_start (GTK_BOX (box), clk, 
 			    FALSE, FALSE, 0); 
 
-	new_game (mfield, NULL);
+	gtk_box_pack_start (GTK_BOX (status_box), box,
+			    FALSE, FALSE, 0);
+
+	new_game ();
 
         gtk_widget_show_all (window);
 
@@ -965,7 +1082,7 @@ main (int argc, char *argv[])
 
 	/* All this hiding is a bit ugly, but it's better than a
 	 * ton of gtk_widget_show calls. */
-	gtk_widget_hide (ralign);
+	gtk_widget_hide (resume_container);
 	gtk_widget_hide (pm_win);
 	gtk_widget_hide (pm_sad);
 	gtk_widget_hide (pm_cool);
